import { createServer } from "http";
import { servelet } from "../servelet.js";
import { allowCors } from "./node-utils/allow-cors.js";
import { healthCheck } from "./node-utils/health-check.js";
import { colorfulLogger } from "../tools/fancy-logging/colorful-logger.js";
import { timestampedLogger } from "../tools/fancy-logging/timestamped-logger.js";
import { makeRequestListener } from "./node-utils/make-request-listener.js";
export function nodeServer({ api, exposeErrors, maxPayloadSize, logger = timestampedLogger(colorfulLogger(console)), processListener = (listener) => listener, }) {
    const execute = servelet(api);
    let listener = makeRequestListener({
        logger,
        exposeErrors,
        maxPayloadSize,
        execute,
    });
    listener = healthCheck("/health", logger, listener);
    listener = allowCors(listener);
    listener = processListener(listener);
    return createServer(listener);
}
//# sourceMappingURL=node-server.js.map