export function colorfulLogger(logger) {
    const codes = {
        reset: "\u001b[0m",
        yellow: "\u001b[33;1m",
        red: "\u001b[31;1m",
    };
    function applyColor(code, items) {
        return items.map(item => typeof item === "string"
            ? code + item + codes.reset
            : item);
    }
    function colorize(log, code) {
        return (...data) => log(...applyColor(code, data));
    }
    return {
        log(...data) {
            logger.log(...data);
        },
        warn: colorize(logger.warn, codes.yellow),
        error: colorize(logger.error, codes.red),
    };
}
//# sourceMappingURL=colorful-logger.js.map